package com.apple.wwbi.TFN.struts2.interceptor;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;

import com.opensymphony.xwork2.ActionInvocation;
import com.opensymphony.xwork2.interceptor.AbstractInterceptor;

public class EJB3Interceptor extends AbstractInterceptor {
	
	private static Map<Class, List<Field>> annotatedMap = new HashMap<Class, List<Field>>();

	@Override
	public String intercept(ActionInvocation actionInvocation) throws Exception {
		Object actionObject = actionInvocation.getAction();
		Class actionClass = actionObject.getClass();
		
		List<Field> annotatedFields = null;
		
		if(annotatedMap.containsKey(actionClass)){
			annotatedFields = annotatedMap.get(actionClass);
		} else {
			annotatedFields = new ArrayList<Field>();
			Field[] fields = actionClass.getDeclaredFields();
			for (Field field : fields) {
				if (field.getAnnotation(EJB.class) != null) {
					annotatedFields.add(field);
				}
			}
			annotatedMap.put(actionClass,annotatedFields);
		}
		
		for (Field field : annotatedFields) {
			if (field.getAnnotation(EJB.class) != null) {
				injectEJB(actionObject,field);
			}
		}
		return actionInvocation.invoke();
	}
	
	private void injectEJB(Object actionObject,Field field){
		EJB ejbAnotation = field.getAnnotation(EJB.class);
		Class ejbClass = (Class)field.getType();
		String beanName = ejbAnotation.name();
		String earName = ejbAnotation.appname();
		String jndiName = earName + "/" + beanName  + "/local";
		//System.out.println("jndiName--------------------"+jndiName);
		//System.out.println("ejbClass---------------------"+ejbClass);
		try {
			Object ejbObject = lookup(ejbClass, jndiName);
			executeSetMethod(field,actionObject,ejbObject,ejbClass);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	private void executeSetMethod(Field field,Object actionObject, Object ejbObejct, Class ejbClass) throws Exception {
		try {
			String methodName = "set" + field.getName().substring(0,1).toUpperCase() + field.getName().substring(1,field.getName().length());
			Method method = actionObject.getClass().getMethod(methodName, new Class[]{ejbClass});
			method.invoke(actionObject, new Object[]{ejbObejct});
		} catch (Exception e) {
			e.printStackTrace();
			throw new Exception(e.getMessage());
		}
	}
	
	private static <T> T lookup(Class<T> ejbClass, String jndiName) throws Exception {
		T t = null;
		try {
			Properties p = new Properties();
			p.setProperty("java.naming.factory.initial", "org.jnp.interfaces.NamingContextFactory");
			//For Local
			//p.setProperty("java.naming.provider.url", "jnp://localhost:1099");
			//For QA
			p.setProperty("java.naming.provider.url", "jnp://localhost:1100");
			p.setProperty("java.naming.factory.url.pkgs", "org.jboss.naming:org.jnp.interfaces");

			Context context = new InitialContext(p);
			return (T)context.lookup(jndiName);
		} catch(Throwable e) {
			e.printStackTrace();
			throw new Exception(e.getMessage());
		}
	}
	
	public static void main(String args[]) throws Exception{
		//lookup(null,"ACGeneratorAdminApp/HelloBean/local");
	}

}
